@@ -23,7 +23,7 @@
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (tm-define (style-category p)
-  (:require (in? p (list "framed-session" "ring-session")))
+  (:require (in? p (list "framed-session" "ring-session" "large-formulas")))
   :session-theme)
 
 (tm-define (style-category-precedes? x y)
@@ -57,6 +57,9 @@
 (tm-define (session-multiline-input?)
   (ahash-ref session-multiline-input (session-key)))
 
+(tm-define (set-session-multiline-input lan ses set?)
+  (ahash-set! session-multiline-input (cons lan ses) set?))
+
 (tm-define (toggle-session-multiline-input)
   (:synopsis "Toggle multi-line input in sessions.")
   (:check-mark "v" session-multiline-input?)
@@ -88,6 +91,16 @@
   (:check-mark "v" session-scheme-trees?)
   (set! session-scheme-trees (not session-scheme-trees)))
 
+(define session-scheme-strees #f)
+
+(tm-define (session-scheme-strees?)
+  session-scheme-strees)
+
+(tm-define (toggle-session-scheme-strees)
+  (:synopsis "Toggle pretty scheme tree output in scheme sessions.")
+  (:check-mark "v" session-scheme-strees?)
+  (set! session-scheme-strees (not session-scheme-strees)))
+
 (define session-scheme-math #f)
 
 (tm-define (session-scheme-math?)
@@ -124,13 +137,18 @@
 (define (error-tree? t)
   (and (tree? t) (tree-is? t 'errput)))
 
-(tm-define (scheme-eval t)
+(tm-define (scheme-eval t mode)
   (let* ((s (texmacs->code t "iso-8859-1"))
 	 (r (eval-string-with-catch s)))
     (cond ((and (tree? r) (error-tree? r) (session-scheme-trees?))
            (tree-copy r))
           ((and (tree? r) (session-scheme-trees?))
            (tree 'text (tree-copy r)))
+          ((and (tm? r) (== mode :silent)
+                (or (session-scheme-trees?) (session-scheme-strees?)))
+           (tree-copy (tm->tree r)))
+          ((and (tm? r) (session-scheme-strees?))
+           (tree 'text (tree-copy (tm->tree r))))
           ((session-scheme-math?)
            (with m (cas->stree r)
                  (if (tm? m) (tree 'math (tm->tree m)) (var-object->string r))))
@@ -169,7 +187,7 @@
 	      (not (session-coherent? out next)))
 	  (plugin-next lan ses)
 	  (begin
-	    (plugin-write lan ses in)
+	    (plugin-write lan ses in :session)
 	    (tree-set out :up 0 (plugin-prompt lan ses)))))))
 
 (define (session-next lan ses)
@@ -752,3 +770,79 @@
 	(tree-insert (tree-ref v (+ j 1)) 0 `(,lan ,ses))
 	(tree-insert v (+ j 1) '((document "")))
 	(tree-go-to v (+ j 1) :end)))))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; Copy and paste
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(define (session-selection-one? t)
+  (and (tree-in? t (cons* 'output 'textput
+                          'folded-subsession 'unfolded-subsession
+                          field-tags))
+       (tree-up t)
+       (session-document-context? (tree-up t))))
+
+(define (session-selection?)
+  (with l (selection-trees)
+    (and (nnull? l) (forall? session-selection-one? l))))
+
+(define (session-selection sel*)
+  (let* ((sel (selection-as-document sel*))
+         (doc (tree-up (tree-ref sel 0)))
+         (ses (tree-up doc)))
+    `(session ,(cDr (tm-children ses)) ,sel)))
+
+(tm-define (clipboard-cut which)
+  (:require (session-selection?))
+  (let* ((l (selection-trees))
+         (doc (tree-up (car l)))
+         (ses (tree-up doc))
+         (i (tree-index (car l)))
+         (j (tree-index (cAr l)))
+         (k (- (+ j 1) i))
+         (n (tree-arity doc)))
+    (clipboard-copy which)
+    (if (= k n)
+        (tree-cut ses)
+        (begin
+          (tree-remove doc i k)
+          (with next (tree-ref doc (min i (- n (+ k 1))))
+            (cond ((field-context? next)
+                   (tree-go-to next 1 :start))
+                  ((tree-in? next '(output textput))
+                   (tree-go-to next 0 :start))
+                  ((tree-in? next '(folded-subsession unfolded-subsession))
+                   (tree-go-to next 0 :start))
+                  (else (tree-go-to next :start))))))))
+
+(tm-define (clipboard-copy which)
+  (:require (session-selection?))
+  (let* ((l (selection-trees))
+         (doc (tree-up (car l)))
+         (ses (tree-up doc))
+         (sel `(session ,@(cDr (tm-children ses)) (document ,@l))))
+    (clipboard-set which sel)))
+
+(tm-define (inside-subsession-context? t)
+  (and (tree-in? t '(folded-subsession unfolded-subsession))
+       (== (tree-arity t) 2)
+       (cursor-inside? (tree-ref t 1))))
+
+(tm-define (clipboard-paste which)
+  (:require (and (inside? 'session)
+                 (tm-ref (clipboard-get which) 1)
+                 (tree-is? (tm-ref (clipboard-get which) 1) 'session)))
+  (let* ((ses (tree-innermost 'session))
+         (sub (tree-innermost inside-subsession-context?))
+         (ins (tree-ref (clipboard-get which) 1)))
+    (when (and (== (tree-arity ses) 3)
+               (== (tree-arity ins) 3)
+               (tm-equal? (tm-ref ses 0) (tm-ref ins 0)))
+      (let* ((doc (if sub (tree-ref sub 1) (tree-ref ses 2)))
+             (ext (tree-ref ins 2))
+             (i (tree-down-index doc)))
+        (if (== (cursor-path) (tree->path doc i :end))
+            (begin
+              (tree-insert doc (+ i 1) (tree-children ext))
+              (tree-go-to doc (+ i (tree-arity ext)) :end))
+            (tree-insert doc i (tree-children ext)))))))
