@@ -76,6 +76,7 @@
 (define-preferences
   ("profile" "beginner" (lambda args (noop)))
   ("look and feel" "default" notify-look-and-feel)
+  ("case sensitive shortcuts" "default" noop)
   ("detailed menus" "detailed" noop)
   ("buffer management" (get-default-buffer-management) notify-buffer-management)
   ("complex actions" "popups" noop)
@@ -112,8 +113,9 @@
   ("experimental alpha" "on" notify-tool)
   ("new style fonts" "on" notify-new-fonts)
   ("bitmap effects" "on" notify-tool)
-  ("new style page breaking" "off" notify-new-page-breaking)
-  ("pattern effects" "off" (lambda args (noop))))
+  ("new style page breaking" "on" notify-new-page-breaking)
+  ("open console on errors" "on" noop)
+  ("open console on warnings" "on" noop))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Properties of some built-in routines
@@ -153,7 +155,7 @@
   (cond ((buffer-embedded? (current-buffer))
          (alt-windows-delete (alt-window-search (current-buffer))))
         ((buffer-modified? (current-buffer))
-         (user-confirm "The buffer has not been saved. Really close it?" #f  
+         (user-confirm "The document has not been saved. Really close it?" #f  
            (lambda (answ)
              (when answ (buffer-close (current-buffer))))))
         (else (buffer-close (current-buffer)))))
@@ -161,7 +163,9 @@
 (define (do-kill-window)
   (with buf (current-buffer)
     (kill-window (current-window))
-    (delayed (:idle 100) (buffer-close buf))))
+    (delayed
+      (:idle 100)
+      (buffer-close buf))))
 
 (tm-define (safely-kill-window . opt-name)
   (cond ((and (buffer-embedded? (current-buffer)) (null? opt-name))
@@ -169,17 +173,27 @@
         ((<= (windows-number) 1)
          (safely-quit-TeXmacs))
         ((nnull? opt-name)
-         (kill-window (car opt-name)))
+         (with buf (window->buffer (car opt-name))
+           (kill-window (car opt-name))
+           (delayed
+             (:idle 100)
+             (buffer-close buf))))
         ((buffer-modified? (current-buffer))
-         (user-confirm "The buffer has not been saved. Really close it?" #f  
+         (user-confirm "The document has not been saved. Really close it?" #f
            (lambda (answ)
              (when answ (do-kill-window)))))
         (else (do-kill-window))))
 
 (tm-define (safely-quit-TeXmacs)
-  (if (not (buffers-modified?)) (quit-TeXmacs)
-      (user-confirm "There are unsaved files. Really quit?" #f  
-        (lambda (answ) (when answ (quit-TeXmacs))))))
+  (with l (filter buffer-modified? (buffer-list))
+    (if (null? l)
+        (quit-TeXmacs)
+        (begin
+          (when (not (buffer-modified? (current-buffer)))
+            ;; FIXME: focus on window with buffer, if any
+            (switch-to-buffer (car l)))
+          (user-confirm "There are unsaved documents. Really quit?" #f  
+            (lambda (answ) (when answ (quit-TeXmacs))))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; System dependent conventions for buffer management
